//
//  TVSSplitViewController.m
//  TableViewSpliter
//
//  Created by narita on 2016/07/29.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "TVSSplitViewController.h"
#import "TVSViewController.h"

@interface TVSSplitViewController ()

@end

@implementation TVSSplitViewController

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark restorableState methods

- (void) encodeRestorableStateWithCoder:(NSCoder *)coder
{
    [super encodeRestorableStateWithCoder:coder];
    
    NSArray* theArray = [self positionsArray];
    
    [coder encodeObject:theArray
                 forKey:@"splitSizes"];
    
    //    NSLog(@"%s, %@ => %@", __func__, @"*theArray*", theArray);
}


- (void) restoreStateWithCoder:(NSCoder *)coder
{
    [super restoreStateWithCoder:coder];

    NSArray* theSplitSizes = [coder decodeObjectForKey:@"splitSizes"];

//    NSLog(@"%s, %@ => %@", __func__, @"*theSplitSizes*", theSplitSizes);

    // 個数分分割して
    NSInteger theSplitCount = theSplitSizes.count;
    
    while(theSplitCount != [self countOfDivider]+1)
    {
        NSSplitViewItem*  theItem = [self makeNewSplitViewItem];
        [self addSplitViewItem:theItem];
    }
    
    // 幅を揃える
    [self setPositionsArray:theSplitSizes];
    
    [self invalidateRestorableState];
}


#pragma mark -
#pragma mark awakeFromNib-like methods

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSSplitViewItem*  theItem = [self makeNewSplitViewItem];
    [self addSplitViewItem:theItem];
}

- (void) viewWillAppear
{
    [super viewWillAppear];
}

- (void) viewDidAppear
{
    [super viewDidAppear];
}

#pragma mark -
#pragma mark action methods

- (NSSplitViewItem*) makeNewSplitViewItem
{
    NSStoryboard*     theStoryboard;
    NSViewController* theViewController;
    NSSplitViewItem*  theItem;
    
    theStoryboard     = [NSStoryboard storyboardWithName:@"MainViewController" bundle:nil];
    theViewController = [theStoryboard instantiateInitialController];
    theItem           = [NSSplitViewItem splitViewItemWithViewController:theViewController];

    return theItem;
}


- (void) insertNewPaneAfter:(TVSViewController*)inViewController
{
    NSSplitViewItem* theCurrentItem = [self splitViewItemForViewController:inViewController];
    NSInteger        theIndex       = [self.splitViewItems indexOfObject:theCurrentItem];
    
    NSSplitViewItem* theNewItem = [self makeNewSplitViewItem];

    // 区切り線の位置を保存して
    NSMutableArray* theArray = [[self positionsArray] mutableCopy];
    
    
    // スクロール位置の調整の準備
    NSScrollView* theCurrentScrollerView = (NSScrollView*)theCurrentItem.viewController.view;
    NSRect        theCurrentRect         = theCurrentScrollerView.contentView.bounds;
    
    // 区切り線を挿入
    [self insertSplitViewItem:theNewItem
                      atIndex:theIndex+1];

    // 挿入後に区切り線の位置を調整して復帰
    CGFloat thePosition_0 = theIndex != 0 ? [[theArray objectAtIndex:theIndex-1] floatValue] : 0;
    CGFloat thePosition_1 = [[theArray objectAtIndex:theIndex] floatValue];
    CGFloat thePosition_2 = roundf((thePosition_1 - thePosition_0)/ 2.0) + thePosition_0;
    [theArray insertObject:@(thePosition_2) atIndex:theIndex];
    
    [self setPositionsArray:theArray];

    
    // 新しく作ったviewのスクロール位置を調整
    NSScrollView* theNewScrollerView = (NSScrollView*)theNewItem.viewController.view;

    NSRect theNewRect = theNewScrollerView.contentView.bounds;
    theNewRect.origin.y = theCurrentRect.origin.y + theCurrentRect.size.height / 2;
    
    // 理由は不明だが、自動でスクロール位置が変更されてしまうために、遅延実行している
    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
        theNewScrollerView.contentView.bounds = theNewRect;
    }];
    
    [self invalidateRestorableState];
}


// 与えられたTSViewControllerを削除する
- (void) removePane:(TVSViewController*)inViewController
{
    if( self.splitViewItems != nil && self.splitViewItems.count > 1)
    {
        NSSplitViewItem* theItem  = [self splitViewItemForViewController:inViewController];
        NSInteger        theIndex = [self.splitViewItems indexOfObject:theItem];

        // 区切り線の位置を保存して
        NSMutableArray* theArray = [[self positionsArray] mutableCopy];

        // theItem.viewControllerがアクティブなレスポンダーチェインに属しているならば、
        // 削除により、firstResponderのポイント先がなくなるので変更する
        BOOL theResetFirstResponder = NO;
        
        NSResponder* theResponder = self.view.window.firstResponder;
        
        while( theResponder )
        {
            if( theResponder == theItem.viewController )
            {
                theResetFirstResponder = YES;
                break;
            }
            
            theResponder = [theResponder nextResponder];
        }        
        
        // TODO:スクロール位置の調整とアニメーション
        [self removeSplitViewItem:theItem];
        
        // 区切り線の位置を調整して復帰
        [theArray removeObjectAtIndex:MAX(0, theIndex-1)];
        [self setPositionsArray:theArray];

        //削除するPaneにフォーカスを変更
        if(theResetFirstResponder)
        {
            [self makeFirstResponderAtIndex:MAX(theIndex-1, 0)];
        }
        
        [self invalidateRestorableState];
    }
}

// 指定したIndexへレスポンダーを移動
- (void) makeFirstResponderAtIndex:(NSInteger)inIndex
{
    NSInteger theIndex = MIN(inIndex, self.splitViewItems.count -1);
    
    NSSplitViewItem* theItem  = [self.splitViewItems objectAtIndex:theIndex];
    
    [self.view.window makeFirstResponder:theItem.viewController.view];
}

#pragma mark -
#pragma mark delegate/datasource methods

// 分割線を移動する時にscrollViewの中身をそれに合わせてスクロールさせる
static CGFloat sOldPosition = 0;

- (void)splitViewWillResizeSubviews:(NSNotification *)notification
{
    NSNumber* theBOOL = [notification.userInfo valueForKey:@"NSSplitViewUserResizeKey"];
    
    //    NSSplitViewUserResizeKey == YESの場合にのみ処理をする
    if( theBOOL.boolValue == YES )
    {
        NSNumber* theIndexObject = [notification.userInfo valueForKey:@"NSSplitViewDividerIndex"];
        NSInteger theIndex = theIndexObject.integerValue;
        sOldPosition = [self positionOfDividerAtIndex:theIndex];
    }
}

- (void)splitViewDidResizeSubviews:(NSNotification *)notification
{
    NSNumber* theBOOL = [notification.userInfo valueForKey:@"NSSplitViewUserResizeKey"];
    
    //    NSSplitViewUserResizeKey == YESの場合にのみ処理をする
    if( theBOOL.boolValue == YES )
    {
        NSNumber* theIndexObject = [notification.userInfo valueForKey:@"NSSplitViewDividerIndex"];
        NSInteger theIndex = theIndexObject.integerValue;
        
        CGFloat theCurrentPosition = [self positionOfDividerAtIndex:theIndex];
    
        NSSplitViewItem* theItem = [self.splitViewItems objectAtIndex:theIndex+1];
        NSScrollView* theScrollerView = (NSScrollView*)theItem.viewController.view;
        
        NSRect theRect = theScrollerView.contentView.bounds;
        theRect.origin.y += theCurrentPosition - sOldPosition;
        
        theScrollerView.contentView.bounds = theRect;
    }
}

#pragma mark -
#pragma mark accessor methods (in pairs)

- (NSArray*) positionsArray
{
    NSMutableArray* theArray = [NSMutableArray array];
    
    for(NSInteger i = 0; i <= [self countOfDivider]; i++)
    {
        CGFloat thePosition = [self positionOfDividerAtIndex:i];
        
        [theArray addObject:@(thePosition)];
    }
    
    return [theArray copy];
}

- (void) setPositionsArray:(NSArray*)inArray
{
    // 幅を揃える
    for(NSInteger i = 0; i < [inArray count]; i++)
    {
        CGFloat thePosition = [[inArray objectAtIndex:i] floatValue];
        
        [self.splitView setPosition:thePosition
                   ofDividerAtIndex:i];
    }
}

#pragma mark -
#pragma mark Utility methods

- (NSInteger) countOfDivider
{
    NSInteger theCount = self.splitViewItems.count;
    
    return (theCount == 0 ) ? 0 : theCount - 1;
}

- (void)setPosition:(CGFloat)position ofDividerAtIndex:(NSInteger)dividerIndex
{
    NSAssert( dividerIndex >= 0, nil);
    NSAssert( dividerIndex < [self countOfDivider], nil);
    
    [self.splitView setPosition:position
               ofDividerAtIndex:dividerIndex];
}

- (CGFloat)positionOfDividerAtIndex:(NSInteger)dividerIndex
{
    NSAssert( dividerIndex >= 0, nil);
    NSAssert( dividerIndex <= [self countOfDivider], nil);
    
    CGFloat thePosition = 0;
    
    for(NSInteger i = 0; i <= dividerIndex; i++)
    {
        NSSplitViewItem* theItem = [self.splitViewItems objectAtIndex:i];
        NSRect theFrame = theItem.viewController.view.frame;
        
        thePosition += theFrame.size.height;
        
        // 2個目のitem以降にはdividerThicknessを含める
        if( i != 0)
        {
            thePosition += self.splitView.dividerThickness;
        }
    }
    
    return thePosition;
}

@end
