//
//  TVScrollView.m
//  TableViewSpliter
//
//  Created by narita on 2016/08/01.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "TVSplitableScrollView.h"
#import "TVSViewController.h"

@interface TVSplitableScrollView ()

@property (strong) NSButton* vSplitOpenButton;
@property (strong) NSButton* vSplitCloseButton;

@end

@implementation TVSplitableScrollView

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

// http://www.cocoabuilder.com/archive/cocoa/284369-how-to-save-and-restore-nsscrollview-scroll-position.html
+ (NSArray *)restorableStateKeyPaths
{
    return [[super restorableStateKeyPaths] arrayByAddingObjectsFromArray:@[ @"contentView.bounds.origin" ]];
}

#pragma mark -
#pragma mark awakeFromNib-like methods

- (void)awakeFromNib
{
    NSAssert(self.viewController != nil, nil);
}

#pragma mark -
#pragma mark action methods

#pragma mark -
#pragma mark delegate/datasource methods


#pragma mark -
#pragma mark accessor methods (in pairs)


#pragma mark -
#pragma mark Utility methods


- (NSButton*) makeSidebarButtonWithImageName:(NSString*)inImageName selector:(SEL)iSEL
{
    NSButton* theResult;
    
    theResult = [[NSButton alloc] initWithFrame: NSZeroRect];
    theResult.buttonType = NSMomentaryLightButton;
    theResult.bezelStyle = NSThickSquareBezelStyle;
    theResult.bordered = NO;
    theResult.image = [NSImage imageNamed:inImageName];
    theResult.imagePosition = NSImageOnly;
    [theResult.cell setImageScaling:NSImageScaleProportionallyDown];
    
    theResult.target = self.viewController;
    theResult.action = iSEL;

    return theResult;
}


- (void)tile
{
    [super tile];
    
    if(  self.hasVerticalScroller)
    {
        NSScroller* vScroller        = self.verticalScroller;
        NSRect      vScrollerFrame   = vScroller.frame;
        
        // 分割ボタン
        if( !self.vSplitOpenButton )
        {
            self.vSplitOpenButton = [self makeSidebarButtonWithImageName:@"splitopen"
                                                                selector:@selector(newPane:)];
            [self addSubview:self.vSplitOpenButton];
        }
        
        NSButton*   vOpenButton      = self.vSplitOpenButton;
        NSRect      vOpenButtonFrame;
        
        // 分割ボタンの位置を設定
        vOpenButtonFrame.origin.x = vScrollerFrame.origin.x;
        vOpenButtonFrame.origin.y = vScrollerFrame.origin.y;
        vOpenButtonFrame.size.height = vScrollerFrame.size.width;
        vOpenButtonFrame.size.width  = vScrollerFrame.size.width;
        vOpenButton.frame = vOpenButtonFrame;
        
        // スクローラーの表示非表示を同期する
        vOpenButton.hidden = vScroller.hidden;
        
        // 結合ボタン
        if( !self.vSplitCloseButton )
        {
            self.vSplitCloseButton = [self makeSidebarButtonWithImageName:@"splitclose"
                                                                 selector:@selector(deletePane:)];
            [self addSubview:self.vSplitCloseButton];
        }
        
        NSButton*   vCloseButton      = self.vSplitCloseButton;
        NSRect      vCloseButtonFrame;
        
        // 結合ボタンの位置を設定
        vCloseButtonFrame.origin.x = vScrollerFrame.origin.x;
        vCloseButtonFrame.origin.y = vScrollerFrame.origin.y + vOpenButtonFrame.size.height;
        vCloseButtonFrame.size.height = vScrollerFrame.size.width;
        vCloseButtonFrame.size.width  = vScrollerFrame.size.width;
        vCloseButton.frame = vCloseButtonFrame;
        
        // スクローラーの表示非表示を同期する
        vCloseButton.hidden = vScroller.hidden;
        
        // スクロールバーの大きさをボタンに合わせて調整
        vScrollerFrame.origin.y += vOpenButtonFrame.size.height + vCloseButtonFrame.size.height;
        vScrollerFrame.size.height -= vOpenButtonFrame.size.height + vCloseButtonFrame.size.height;
        [vScroller setFrame:vScrollerFrame];
    }
}

@end
